/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filter;

//---IMPORTS---
import java.io.*;
import java.net.*;
import java.util.StringTokenizer;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.filter.*;
import uk.org.iscream.cms.util.*;

/**
 * Handle an incoming packet as a separate thread.
 * Passes the data through various plugins, then 
 * passes it on to the parent filter.
 *
 * Now grabs data from a single queue, rather than
 * waiting to be contacted.
 *
 * @author  $Author: tdb $
 * @version $Id: FilterThread.java,v 1.41 2005/09/25 09:57:42 tdb Exp $
 */
public class FilterThread extends Thread{

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.41 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
   
    /**
     * Constructs an instance of a FilterThread
     *
     * @param queue the Queue this filter is using
     */
    public FilterThread(Queue queue){
        // set the Thread name
        setName("filter.FilterThread");
        
        _queue = queue;
        _logger.write(toString(), Logger.SYSINIT, "created");
    }

//---PUBLIC METHODS---

    /**
     * Runs the thread, getting data from the Queue and
     * sending it on to the parent filter.
     */
    public void run(){
        // get a queue for ourselves
        int n = _queue.getQueue();
        // keep these out here, saves recreating the object
        String xml = null;
        String parentFilterName = "";
        Filter parent = null;
        while(true) {
            // get a String of xml
            try {
                xml = (String) _queue.get(n);
            }
            catch (InvalidQueueException e) {
                _logger.write(toString(), Logger.ERROR, "Queue error: "+e);
            }
            
            // Get a string without any null characters in it.
            //  -- maybe String.trim() would be better here ?
            if (xml.indexOf(0) != -1) {
                xml = xml.substring(0, xml.indexOf(0));
            }
            
            // Bundle the XML all on one line (saves space and simplifies
            // the protocol between clientinterface and client).
            StringTokenizer tokenizer = new StringTokenizer(new String(xml), "\n");
            xml = "";
            while (tokenizer.hasMoreTokens()) {
                xml += tokenizer.nextToken();
            }
            
            // Get an XMLPacket from the XMLCache.
            XMLPacket packet = null;
            try {
                packet = _xmlCache.getXMLPacket(xml);
            } catch(InvalidXMLException e) {
                _logger.write(toString(), Logger.ERROR, "Invalid XML: "+e);
                // skip the rest of this loop iteration
                continue;
            }
            
            // get parent
            try {
                String newParent = ConfigurationProxy.getInstance().getProperty("Filter." + FilterMain.NAME, "Filter.parentFilter");
                if(!parentFilterName.equals(newParent)) {
                    parentFilterName = newParent;
                    parent = FilterHelper.narrow(_refman.getCORBARef("iscream.Filter\\." + parentFilterName));
                    _logger.write(toString(), Logger.DEBUG, "Parent filter changed to: "+parentFilterName);
                }
            } catch (PropertyNotFoundException e) {
                continue;
            }
                    
            // XMLPacket is ok, run filters...
            if(PluginFilterManager.getInstance().runFilters(packet)) {
                // and pass it on...
                parent.receiveXML(xml);
            }
            else {
                // ... or filtered it
                _logger.write(toString(), Logger.DEBUG, "An XML packet was sucessfully filtered from the system.");
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
    
    /**
     * The Queue object
     */
    Queue _queue;
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * A reference to the XMLCache in use
     */
    private XMLCache _xmlCache = XMLCache.getInstance();

//---STATIC ATTRIBUTES---

}
