/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.rootfilter;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.clientinterface.*;


/**
 * Acts as a Control Handler to a CORBA based client.
 *
 * @author  $Author: tdb $
 * @version $Id: CorbaControlHandlerServant.java,v 1.2 2004/08/01 10:41:06 tdb Exp $
 */
class CorbaControlHandlerServant extends CorbaControlHandlerPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.2 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Construct a new CorbaControlHandlerServant.
     *
     * @param queue A reference to the Queue we're using
     * @param client A reference to the "servant" part of the connecting client.
     * @param clientname A name to identify the client.
     */
    public CorbaControlHandlerServant(Queue queue, Client client, String clientname) {
        _queue = queue;
        _client = client;
        _clientname = clientname;
        _dataHandler = null;
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
    
//---PUBLIC METHODS---
    
    /**
     * Start sending data to the client.
     *
     * @return a boolean stating whether the attempt to start succeeded
     */
    public boolean startData() {
        if(_dataHandler == null) {
            // create a new DataHandler
            CorbaDataHandler dh = new CorbaDataHandler(_client, this, _queue);
            // start the DataHandler running
            dh.start();
            // keep a reference
            _dataHandler = dh;
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Stop sending data to the client.
     *
     * @return a boolean stating whether the attempt to stop succeeded
     */
    public boolean stopData() {
        if(_dataHandler != null) {
            // stop the DataHandler
            _dataHandler.shutdown();
            // destroy the reference
            _dataHandler = null;
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Sets the host list for the Client to the requested semi-colon separated
     * list of hostnames. This will only succeed if we are not sending data.
     *
     * @param hostList A semi-colon separated list of hostnames to use.
     * @return Whether the request succeeded.
     */
    public boolean setHostList(String hostList) {
        return false;
    }
    
    /**
     * Disconnect, this will shutdown the data and unhook from
     * the CORBA ORB.
     */
    public void disconnect() {
        // close the data handler
        stopData();
        // disconnect from the ORB
        try {
            byte[] oid = _refman.getRootPOA().servant_to_id(this);
            _refman.getRootPOA().deactivate_object(oid);
        } catch(Exception e) {
            _logger.write(this.toString(), Logger.ERROR, "disconnect failed: "+e);
        }
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * Overridden for debugging purposes
     * to see when an instance of this class
     * is destroyed
     */
    protected void finalize() throws Throwable {
        _logger.write(this.toString(), Logger.DEBUG, "finalized by GC");
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = RootFilter.NAME;
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * A reference to our Queue.
     */
    private Queue _queue;
    
    /**
     * The "servant" part of the client we're connected to.
     */
    private Client _client;
    
    /**
     * A reference to our DataHandler, if we have one
     */
    private CorbaDataHandler _dataHandler;
    
    /**
     * A name to identify the client
     */
    private String _clientname;
    
//---STATIC ATTRIBUTES---

}
